// ignore: avoid_web_libraries_in_flutter
import 'dart:html';
// ignore: avoid_web_libraries_in_flutter
import 'dart:js' as js;
import 'dart:convert';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';

typedef NUICrossEventMessageListener = void Function(NUIMiniProgramMessage message);

class NUIMiniProgramWebJSEngine{
  NUIMiniProgramWebJSEngine._builder();
  Window _window;

  static NUIMiniProgramWebJSEngine from(Window window){
    return NUIMiniProgramWebJSEngine._builder().._window = window;
  }

  void initCrossEventListener(NUICrossEventMessageListener listener){
    _window.addEventListener("message", (ev){
      MessageEvent messageEvent = ev;
      print("[NUIMiniProgramJSEngine] Message Event: ${messageEvent.data}, window name: ${window.name}");

      final message = _processEventMessage(messageEvent.data);
      if(message != null){
        listener(message);
      }
    });
  }

  bool sendCrossEvent(NUIMiniProgramMessage message){
    try {
      js.context.callMethod('postNUIMessage', [jsonEncode(message)]);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to post NUI Message with error : $e");
      return false;
    }
  }

  bool sendPopEvent(){
    final internalMessage = NUIMiniProgramInternalMessage(type: NUIMiniProgramEventType.POP, data: "pop");
    final popMessage = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      js.context.callMethod('postNUIMessage', [jsonEncode(popMessage)]);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to post NUI Message with error : $e");
      return false;
    }
  }

  NUIMiniProgramMessage _processEventMessage(String message){
    try{
      final decoded = jsonDecode(message);
      final programMessage = NUIMiniProgramMessage().fromJson(decoded);
      if(programMessage.channel == null || programMessage.data == null){
        return null;
      }
      return programMessage;
    }catch(e){
      return null;
    }
  }

}